Manual do Blem
==============

O `blem` é um conjunto de scripts Python para ajudar a criar problemas.
Ele foi desenvolvido por Fernando Monteiro Kiotheka.

Instruções
----------

O script `blem` é encontrado no diretório `blem` do diretório raiz do repositório.
Você pode incluir o caminho para o diretório através do script `blem/activate`,
o chamando dentro do seu shell com `source blem/activate`.

Dentro da raiz do diretório você pode criar um problema novo pelo comando
`blem new [NOME]` com o nome `NOME`. Isso irá criar o esqueleto de um novo
problema.

Para mexer em um determinado problema você precisa ou estar no diretório dele
ou especificar o diretório do problema. Um problema é identificado pelo
arquivo `blem.toml` e o `blem` irá percorrer alguns diretórios para cima até
encontrar um problema. Um diretório de problema é composto de uma estrutura como segue:

- `artifacts/`: Diretório de compilação com os casos de teste gerados e informações
de execuções prévias do `blem`. Todo o conteúdo deste diretório pode ser apagado
e ele é automaticamente criado se necessário.
- `blem`: Este é um link simbólico para o diretório `blem` na raiz do repositório,
ou uma cópia do script do `blem` completo via subrepositório, caso o repositório
seja composto de apenas um problema. Ele permite que você utilize por exemplo
a `testlib.h` incluída no `blem` de forma relativa: basta criar um link
simbólico para `../blem/testlib/testlib.h` no diretório de validadores por exemplo.
- `blem.toml`: Um arquivo com os metadados do problema. Você pode configurar
os autores, o tempo limite em segundos e o nome do problema em várias linguagens:
   ```toml
   authors = ["Me", "Myself", "I"]

   time_limit_ms = 1000

   [br]
   name = "Nome"

   [en]
   name = "Name"
   ```
- `evaluator/`: Diretório com o código utilizado para julgar se uma solução
é considerada correta ou não. Dentro deste diretório crie um arquivo chamado
`evaluator` na linguagem que desejar como C++ ou Python (com extensão `.cpp`
ou `.py`) e o `blem` irá automaticamente compilá-lo quando necessário. Apenas
é necessário escrever este código se o problema tiver saídas válidas especiais,
pois a testlib conta com vários programas chamados de *checkers* que fazem este
trabalho. Eles podem ser encontrados em `../blem/testlib/checkers/`. Por exemplo
criar um link simbólico `evaluator.cpp` para `../blem/testlib/checkers/rcmp6.cpp`
e `testlib.h` para `../blem/testlib/testlib.h` é suficiente para que as respostas
sejam consideradas corretas mesmo com erro relativo ou absoluto de $10^{-6}$.
- `solutions/`: Diretório com as soluções do problema. A definição do veredito
esperado de uma solução é definida pelo seu sufixo. Um sufixo `-ac` como em
`fernando-ac.cpp` determina que a solução tem que receber o veredito de resposta
correta. Outros vereditos possíveis são: `-wa` (resposta errada em pelo menos
um caso de teste), `-tl` (tempo excedido em pelo menos um caso de teste, mas correto
em outros), `-re` (erro de execução em pelo menos um caso de teste),
`-ml` (limite de memória excedido em pelo menos um caso de teste, mas correto em
outros). Podem ser script programas em Python e C++.
- `statement/`: Diretório com os enunciados em várias linguagens. O padrão
é que os enunciados sejam escritos em Markdown, com extensão `.md`, onde sintaxe
de LaTeX com `$` é suportada. As subseções de **Entrada** e **Saída** devem
ser prefixadas com três `#`s, ou seja, sáo seçöes `<h3>`. O `blem` é responsável
por gerar versões em HTML e PDF do enunciado através das ferramentas Pandoc e LaTeX.
A diretiva `\example{0-sample/1}` em um parágrafo sozinho adiciona o primeiro
caso de teste do grupo de casos de teste `0-sample` como exemplo.
A explicação de um caso de teste pode ser adicionada após, em um parágrafo separado.
- `testcases/`: Diretório com os geradores de casos de teste, exemplos e especificação
dos casos de teste. É recomendado que você tenha um symlink para a `testlib.h`,
criando um symlink para `../blem/testlib/testlib.h` para fazer os seus próprios
geradores de caso de teste. Neste diretório você encontra o arquivo `testplan.txt`
que especifica com os casos de teste devem ser gerados. Um `testplan.txt` de
exemplo é:

  ```
  group 0-sample
  copy sample-1.in
  copy sample-2.in

  group 1-random
  ./gen_random 1 1
  ```

  O primeiro grupo é sempre o grupo de exemplos, e os grupos de casos de teste
são especificados pelo comando `group` seguido do nome. Existem dois tipos de
caso de teste: os manuais, especificados com o comando `copy` seguido do arquivo
a ser copiado; e gerados, especificados pela execução de um comando com a
sintaxe `./comando` seguido dos argumentos do comando. As respostas serão
geradas automaticamente através de uma solução `-main-ac`.
- `validator/`: Diretorio com o validador de casos de teste. Também recomenda-se
ter um symlink para a `testlib.h`. Você deve escrever um programa `validator`
que lê da entrada padrão um caso de teste e verifica se ele está adequado ao
enunciado. Assim, os casos de teste podem ser verificados pela sua corretude
em relação ao enunciado, além de ser possívei identificar alguns tipos de
caso de teste omissos (como falha em explorar todas os limites e afins).

Para manejar os problemas existem alguns comandos (em chaves os argumentos são
opcionais):

- `blem new NAME`: Cria um novo problema com o nome especificado.
- `blem status [PROBLEM]`: Verifica o estado do problema em relação a estrutura do
seu diretório e validações previamente executadas.
- `blem make-output [PROBLEM]`: Compila todos as soluções e seleciona uma das soluções
aceitas para gerar as saídas do problema no diretório `artifacts/`.
- `blem run SOURCES...`: Testa os arquivos dados como argumento em relação aos casos de
teste do seus respectivos problemas.
- `blem make-statement [PROBLEM] [LANGUAGE]`: Gera versões em HTML e PDF dos enunciados em `.md`.
- `blem validate-tests [PROBLEM]`: Valida os testes usando o `validator`.
