local system = require 'pandoc.system'

local tikz_doc_template = [[
\documentclass{standalone}
\usepackage{xcolor}
\usepackage{tikz}
\begin{document}
\nopagecolor
%s
\end{document}
]]

local function file_exists(name)
  local f = io.open(name, 'r')
  if f ~= nil then
    io.close(f)
    return true
  else
    return false
  end
end

local function tikz2image(src, filetype, outfile)
  system.with_temporary_directory('tikz2image', function (tmpdir)
    system.with_working_directory(tmpdir, function()
      local f = io.open('tikz.tex', 'w')
      f:write(tikz_doc_template:format(src))
      f:close()
      os.execute('pdflatex tikz.tex')
      if filetype == 'pdf' then
        os.execute('mv -- tikz.pdf "' .. outfile .. '"')
      else
        os.execute('pdf2svg tikz.pdf tikz.svg')
        inp = io.open('tikz.svg', 'r')
        contents = inp:read('*all')
        contents = contents:gsub('width="([0-9.]-)" height="([0-9.]-)" ', function(w, h)
          return 'width="' .. w .. 'pt" height="' .. h .. 'pt" '
        end)
        inp:close()
        out = io.open(outfile, 'w')
        out:write(contents)
        out:close()
      end
    end)
  end)
end

extension_for = {
  html = 'svg',
  html4 = 'svg',
  html5 = 'svg',
  latex = 'pdf',
  beamer = 'pdf' }

local function starts_with(start, str)
  return str:sub(1, #start) == start
end


function RawBlock(el)
  if starts_with('\\begin{tikzpicture}', el.text) then
    local filetype = extension_for[FORMAT] or 'svg'
    local fbasename = pandoc.sha1(el.text) .. '.gen.' .. filetype
    local fname = PANDOC_STATE.output_file:match("(.*/)") .. fbasename
    if not file_exists(fname) then
      tikz2image(el.text, filetype, fname)
    end
    return pandoc.Image({}, fname)
  else
   return el
  end
end
